@@ -20,8 +20,10 @@ import simplejson
 
 from zodbbrowser import __version__, __homepage__
 from zodbbrowser.interfaces import IObjectHistory, IValueRenderer
+from zodbbrowser.interfaces import IDatabaseHistory
 from zodbbrowser.state import ZodbObjectState
 from zodbbrowser.diff import compareDictsHTML
+from zodbbrowser.value import pruneTruncations, TRUNCATIONS
 
 
 log = logging.getLogger("zodbbrowser")
@@ -61,7 +63,48 @@ class ZodbObjectAttribute(object):
         return not self.__eq__(other)
 
 
-class ZodbInfoView(BrowserView):
+class VeryCarefulView(BrowserView):
+
+    made_changes = False
+
+    @Lazy
+    def jar(self):
+        db = queryUtility(IDatabase, name='<target>')
+        if db is not None:
+            conn = db.open()
+            self.request.hold(Cleanup(conn.close))
+            return conn
+        try:
+            return self.request.annotations['ZODB.interfaces.IConnection']
+        except (KeyError, AttributeError):
+            obj = self.findClosestPersistent()
+            if obj is None:
+                raise Exception("ZODB connection not available for this request")
+            return obj._p_jar
+
+    @property
+    def readonly(self):
+        return self.jar.isReadOnly()
+
+    def __call__(self):
+        try:
+            return self.render()
+        finally:
+            if self.readonly or not self.made_changes:
+                resources = transaction.get()._resources
+                if resources:
+                    msg = ["Aborting changes made to:"]
+                    for r in resources:
+                        if isinstance(r, Connection):
+                            for o in r._registered_objects:
+                                msg.append("  oid=%s %s" % (oid_repr(o._p_oid), repr(o)))
+                        else:
+                            msg.append("  %s" % repr(r))
+                    log.debug("\n".join(msg))
+                transaction.abort()
+
+
+class ZodbInfoView(VeryCarefulView):
     """Zodb browser view"""
 
     adapts(Interface, IBrowserRequest)
@@ -72,29 +115,9 @@ class ZodbInfoView(BrowserView):
     version = __version__
     homepage = __homepage__
 
-    @property
-    def readonly(self):
-        jar = self.jar
-        return jar.isReadOnly()
-
-    def __call__(self):
-        try:
-            return self.render()
-        finally:
-            if self.readonly:
-                resources = transaction.get()._resources
-                if resources:
-                    msg = ["Aborting changes made to:"]
-                    for r in resources:
-                        if isinstance(r, Connection):
-                            for o in r._registered_objects:
-                                msg.append("  oid=%s %s" % (oid_repr(o._p_oid), repr(o)))
-                        else:
-                            msg.append("  %s" % repr(r))
-                    log.warning("\n".join(msg))
-                transaction.abort()
-
     def render(self):
+        self._started = time.time()
+        pruneTruncations()
         self.obj = self.selectObjectToView()
         self.history = IObjectHistory(self.obj)
         self.latest = True
@@ -117,6 +140,7 @@ class ZodbInfoView(BrowserView):
                 self.history.rollback(rtid)
                 transaction.get().note('Rollback to old state %s'
                                         % self.requestedState)
+                self.made_changes = True
                 self._redirectToSelf()
                 return ''
             # will show confirmation prompt
@@ -124,6 +148,9 @@ class ZodbInfoView(BrowserView):
 
         return self.template()
 
+    def renderingTime(self):
+        return '%.3fs |' % (time.time() - self._started)
+
     def _redirectToSelf(self):
         self.request.response.redirect(self.getUrl())
 
@@ -169,11 +196,10 @@ class ZodbInfoView(BrowserView):
         return self.state.getObjectId()
 
     def getObjectType(self):
-        cls = getattr(self.obj, '__class__', None)
-        if type(self.obj) is not cls:
-            return '%s - %s' % (type(self.obj), cls)
-        else:
-            return str(cls)
+        return getObjectType(self.obj)
+
+    def getObjectTypeShort(self):
+        return getObjectTypeShort(self.obj)
 
     def getStateTid(self):
         return u64(self.state.tid)
@@ -181,6 +207,9 @@ class ZodbInfoView(BrowserView):
     def getStateTidNice(self):
         return self._tidToTimestamp(self.state.tid)
 
+    def getPickleSize(self):
+        return len(self.state.pickledState)
+
     def getRootOid(self):
         root = self.jar.root()
         try:
@@ -189,23 +218,11 @@ class ZodbInfoView(BrowserView):
             pass
         return u64(root._p_oid)
 
-    def locate_json(self, path):
+    def locate_json(self, path): # AJAX view
         return simplejson.dumps(self.locate(path))
 
-    @Lazy
-    def jar(self):
-        db = queryUtility(IDatabase, name='<target>')
-        if db is not None:
-            conn = db.open()
-            self.request.hold(Cleanup(conn.close))
-            return conn
-        try:
-            return self.request.annotations['ZODB.interfaces.IConnection']
-        except KeyError:
-            obj = self.findClosestPersistent()
-            if obj is None:
-                raise Exception("ZODB connection not available for this request")
-            return obj._p_jar
+    def truncated_ajax(self, id): # AJAX view
+        return TRUNCATIONS.get(id)
 
     def locate(self, path):
         not_found = object() # marker
@@ -377,3 +394,142 @@ class ZodbInfoView(BrowserView):
         if isinstance(tid, str) and len(tid) == 8:
             return str(TimeStamp(tid))
         return tid_repr(tid)
+
+
+class ZodbHistoryView(VeryCarefulView):
+    """Zodb history view"""
+
+    adapts(Interface, IBrowserRequest)
+
+    template = ViewPageTemplateFile('templates/zodbhistory.pt')
+
+    version = __version__
+    homepage = __homepage__
+
+    page_size = 5
+
+    def render(self):
+        self._started = time.time()
+        pruneTruncations()
+        if 'page_size' in self.request:
+            self.page_size = max(1, int(self.request['page_size']))
+        self.history = IDatabaseHistory(self.jar)
+        if 'page' in self.request:
+            self.page = int(self.request['page'])
+        elif 'tid' in self.request:
+            tid = int(self.request['tid'], 0)
+            self.page = self.findPage(p64(tid))
+        else:
+            self.page = 0
+        self.last_page = max(0, len(self.history) - 1) // self.page_size
+        if self.page > self.last_page:
+            self.page = self.last_page
+        self.last_idx = max(0, len(self.history) - self.page * self.page_size)
+        self.first_idx = max(0, self.last_idx  - self.page_size)
+        return self.template()
+
+    def renderingTime(self):
+        return '%.3fs |' % (time.time() - self._started)
+
+    def getUrl(self, tid=None):
+        url = "@@zodbbrowser_history"
+        if tid is None and 'tid' in self.request:
+            url += "?tid=" + self.request['tid']
+        elif tid is not None:
+            url += "?tid=0x%x" % tid
+        return url
+
+    def findPage(self, tid):
+        try:
+            pos = list(self.history.tids).index(tid)
+        except ValueError:
+            return 0
+        else:
+            return (len(self.history) - pos - 1) // self.page_size
+
+    def listHistory(self):
+        if 'tid' in self.request:
+            requested_tid = p64(int(self.request['tid'], 0))
+        else:
+            requested_tid = None
+
+        results = []
+        for n, d in enumerate(self.history[self.first_idx:self.last_idx]):
+            utid = u64(d.tid)
+            short = "%s %s %s" % (TimeStamp(d.tid),
+                                  d.user,
+                                  d.description)
+            objects = []
+            for record in d:
+                obj = self.jar.get(record.oid)
+                url = "@@zodbbrowser?oid=0x%x&tid=0x%x" % (u64(record.oid),
+                                                           utid)
+                objects.append(dict(
+                    oid=u64(record.oid),
+                    path=getObjectPath(obj, d.tid),
+                    oid_repr=oid_repr(record.oid),
+                    class_repr=getObjectType(obj),
+                    url=url,
+                    repr=IValueRenderer(obj).render(d.tid),
+                ))
+            if len(objects) == 1:
+                summary = '1 object record'
+            else:
+                summary = '%d object records' % len(objects)
+            results.append(dict(
+                index=(self.first_idx + n + 1),
+                short=short,
+                utid=utid,
+                current=(d.tid == requested_tid),
+                href=self.getUrl(tid=utid),
+                summary=summary,
+                hidden=(len(objects) > 5),
+                objects=objects,
+            ))
+        if results and not requested_tid and self.page == 0:
+            results[-1]['current'] = True
+        return results[::-1]
+
+
+def getObjectType(obj):
+    cls = getattr(obj, '__class__', None)
+    if type(obj) is not cls:
+        return '%s - %s' % (type(obj), cls)
+    else:
+        return str(cls)
+
+
+def getObjectTypeShort(obj):
+    cls = getattr(obj, '__class__', None)
+    if type(obj) is not cls:
+        return '%s - %s' % (type(obj).__name__, cls.__name__)
+    else:
+        return cls.__name__
+
+
+def getObjectPath(obj, tid):
+    path = []
+    seen_root = False
+    state = ZodbObjectState(obj, tid)
+    while True:
+        if state.isRoot():
+            path.append('/')
+            seen_root = True
+        else:
+            if path:
+                path.append('/')
+            if not state.getName() and state.getParentState() is None:
+                # not using hex() because we don't want L suffixes for
+                # 64-bit values
+                path.append('0x%x' % state.getObjectId())
+                break
+            path.append(state.getName() or '???')
+        state = state.getParentState()
+        if state is None:
+            if not seen_root:
+                path.append('/')
+                path.append('...')
+                path.append('/')
+            break
+    return ''.join(path[::-1])
+
