@@ -12,9 +12,12 @@ from zope.component import provideAdapter, getGlobalSiteManager
 from zope.interface import implements
 from zope.publisher.browser import TestRequest
 from zope.traversing.interfaces import IContainmentRoot
+from zope.security.checker import ProxyFactory
 
 from zodbbrowser.state import GenericState, ZodbObjectState
 from zodbbrowser.browser import ZodbObjectAttribute, ZodbInfoView
+from zodbbrowser.browser import getObjectType, getObjectTypeShort
+from zodbbrowser.browser import getObjectPath
 from zodbbrowser.history import ZodbObjectHistory
 from zodbbrowser.testing import SimpleValueRenderer
 
@@ -163,7 +166,8 @@ class TestZodbInfoViewWithRealDb(RealDatabaseTest):
 
     def testSelectObjectToView_by_oid_in_hex(self):
         oid = u64(self.root['stub']._p_oid)
-        view = ZodbInfoView(self.root, TestRequest(form={'oid': hex(oid)}))
+        hex_oid = hex(oid).rstrip('L')
+        view = ZodbInfoView(self.root, TestRequest(form={'oid': hex_oid}))
         self.assertEquals(view.selectObjectToView(), self.root['stub'])
 
     def testFindClosestPersistent(self):
@@ -446,6 +450,22 @@ class TestZodbInfoView(unittest.TestCase):
                           "'something else'")
 
 
+class TestHelperFunctions(unittest.TestCase):
+
+    def test_getObjectType(self):
+        self.assertEqual(getObjectType(NonpersistentStub()),
+                         "<class 'zodbbrowser.tests.test_browser.NonpersistentStub'>")
+        self.assertEqual(getObjectType(ProxyFactory(NonpersistentStub())),
+                         "<type 'zope.security._proxy._Proxy'> -"
+                         " <class 'zodbbrowser.tests.test_browser.NonpersistentStub'>")
+
+    def test_getObjectTypeShort(self):
+        self.assertEqual(getObjectTypeShort(NonpersistentStub()),
+                         'NonpersistentStub')
+        self.assertEqual(getObjectTypeShort(ProxyFactory(NonpersistentStub())),
+                         '_Proxy - NonpersistentStub')
+
+
 def test_suite():
     this = sys.modules[__name__]
     return unittest.TestSuite([
